@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use ansi_term::Color::{self, Yellow, Blue, Green, Red, Purple, Fixed};
+use ansi_term::Color;
 use bstr::ByteSlice;
 use reduce::Reduce;
 use sha2::{Sha512, Digest};
@@ -8,7 +8,7 @@ use std::cmp;
 use std::fmt::Debug;
 use pktparse::icmp::{IcmpHeader, IcmpCode, IcmpData};
 
-use crate::structs::ether;
+use crate::structs::ether::Ether;
 use crate::structs::arp;
 use crate::structs::cjdns;
 use crate::structs::ip::IPHeader;
@@ -17,12 +17,12 @@ use crate::structs::ipv6;
 use crate::structs::tcp;
 use crate::structs::udp;
 use crate::structs::icmp;
-use crate::structs::tls;
+use crate::structs::http;
 use crate::structs::raw::Raw;
-use crate::structs::prelude::*;
+use crate::structs::tls;
 use crate::structs::NoiseLevel;
 
-const GREY: u8 = 245;
+const GREY: Color = Color::Fixed(245);
 
 
 pub struct Config {
@@ -88,18 +88,17 @@ impl Format {
     fn print_compact(&self, packet: Raw) {
         let mut out = String::new();
 
-        use crate::structs::raw::Raw::Unknown;
         let color = match packet {
-            Ether(eth_frame, eth) => {
+            Raw::Ether(eth_frame, eth) => {
                 out += &format!("{} -> {}, ",
                                 display_macaddr(eth_frame.source_mac),
                                 display_macaddr(eth_frame.dest_mac));
 
                 self.format_compact_eth(&mut out, eth)
             },
-            Tun(eth) => self.format_compact_eth(&mut out, eth),
-            Sll(eth) => self.format_compact_eth(&mut out, eth),
-            Unknown(data) => self.format_compact_unknown_data(&mut out, &data),
+            Raw::Tun(eth) => self.format_compact_eth(&mut out, eth),
+            Raw::Sll(eth) => self.format_compact_eth(&mut out, eth),
+            Raw::Unknown(data) => self.format_compact_unknown_data(&mut out, &data),
         };
 
         println!("{}", match color {
@@ -115,13 +114,13 @@ impl Format {
     }
 
     #[inline]
-    fn format_compact_eth(&self, out: &mut String, eth: ether::Ether) -> Option<Color> {
+    fn format_compact_eth(&self, out: &mut String, eth: Ether) -> Option<Color> {
         match eth {
-            Arp(arp_pkt) => Some(self.format_compact_arp(out, &arp_pkt)),
-            IPv4(ip_hdr, ipv4) => self.format_compact_ipv4(out, &ip_hdr, ipv4),
-            IPv6(ip_hdr, ipv6) => self.format_compact_ipv6(out, &ip_hdr, ipv6),
-            Cjdns(cjdns_pkt) => Some(self.format_compact_cjdns(out, &cjdns_pkt)),
-            ether::Ether::Unknown(data) => self.format_compact_unknown_data(out, &data),
+            Ether::Arp(arp_pkt) => Some(self.format_compact_arp(out, &arp_pkt)),
+            Ether::IPv4(ip_hdr, ipv4) => self.format_compact_ipv4(out, &ip_hdr, ipv4),
+            Ether::IPv6(ip_hdr, ipv6) => self.format_compact_ipv6(out, &ip_hdr, ipv6),
+            Ether::Cjdns(cjdns_pkt) => Some(self.format_compact_cjdns(out, &cjdns_pkt)),
+            Ether::Unknown(data) => self.format_compact_unknown_data(out, &data),
         }
     }
 
@@ -143,7 +142,7 @@ impl Format {
                     display_macaddr(arp_pkt.dest_mac))
             },
         });
-        Blue
+        Color::Blue
     }
 
     #[inline]
@@ -156,7 +155,7 @@ impl Format {
 
         let ipv6 = {
             let bytes1 = Sha512::digest(&cjdns.pubkey);
-            let bytes2 = Sha512::digest(&bytes1);
+            let bytes2 = Sha512::digest(bytes1);
 
             let mut iter = bytes2.as_slice().iter();
 
@@ -181,7 +180,7 @@ impl Format {
                               ipv6,
                               cjdns.pubkey));
 
-        Purple
+        Color::Purple
     }
 
     #[inline]
@@ -226,11 +225,44 @@ impl Format {
 
         use crate::structs::tcp::TCP::*;
         match tcp {
-            HTTP(http) => {
-                // println!("{}", Green.normal().paint(format!("\t\t\thttp: {:?} {:?}", format!("{} http://{}{} HTTP/{}", http.method, http.host.clone().unwrap_or("???".to_owned()), http.uri, http.version), http)));
-                out.push_str(&format!("[http] {:?}", http)); // TODO
-                Green
+            HTTP(http::Http::Request(http)) => {
+                out.push_str("[http] req, ");
+
+                let offset = out.len();
+                out.push_str(&format!("{:?} {:?} HTTP/1.{}", http.method, http.path, http.version));
+
+                if let Some(host) = &http.host {
+                    out.push_str(&format!(" - http://{host}{}", http.path));
+                }
+
+                for (key, value) in &http.headers {
+                    out.push_str(&align(offset, &format!("{key:?}: {value:?}")));
+                }
+
+                if let Some(body) = http.body {
+                    out.push('\n');
+                    out.push_str(&align(offset, &format!("{body:?}")));
+                }
+
+                Color::Red
             },
+            HTTP(http::Http::Response(http)) => {
+                out.push_str("[http] resp, ");
+
+                let offset = out.len();
+                out.push_str(&format!("HTTP/1.{} {} {:?} ", http.version, http.code, http.reason));
+
+                for (key, value) in &http.headers {
+                    out.push_str(&align(offset, &format!("{key:?}: {value:?}")));
+                }
+
+                if let Some(body) = http.body {
+                    out.push('\n');
+                    out.push_str(&align(offset, &format!("{body:?}")));
+                }
+
+                Color::Red
+            }
             TLS(tls::TLS::ClientHello(client_hello)) => {
                 let extra = display_kv_list(&[
                     ("version", client_hello.version),
@@ -240,7 +272,7 @@ impl Format {
 
                 out.push_str("[tls] ClientHello");
                 out.push_str(&extra);
-                Green
+                Color::Green
             },
             TLS(tls::TLS::ServerHello(server_hello)) => {
                 let extra = display_kv_list(&[
@@ -251,18 +283,22 @@ impl Format {
 
                 out.push_str("[tls] ServerHello");
                 out.push_str(&extra);
-                Green
+                Color::Green
             },
             Text(text) => {
                 out.push_str(&format!("[text] {:?}", text));
-                Red
+                Color::Red
             },
             Binary(x) => {
                 out.push_str(&format!("[binary] {:?}", x.as_bstr()));
-                Red
+                if tcp_hdr.flag_rst {
+                    GREY
+                } else {
+                    Color::Red
+                }
             },
             Empty => {
-                Fixed(GREY)
+                GREY
             },
         }
     }
@@ -320,7 +356,7 @@ impl Format {
                     },
                 };
 
-                Blue
+                Color::Blue
             },
             DNS(dns) => {
                 use crate::structs::dns::DNS::*;
@@ -347,7 +383,7 @@ impl Format {
                     },
                 };
 
-                Yellow
+                Color::Yellow
             },
             SSDP(ssdp) => {
                 use crate::structs::ssdp::SSDP::*;
@@ -357,7 +393,7 @@ impl Format {
                     Notify(extra) => format!("[ssdp] notify: {:?}", extra),
                     BTSearch(extra) => format!("[ssdp] torrent search: {:?}", extra),
                 });
-                Purple
+                Color::Purple
             },
             Dropbox(dropbox) => {
                 out.push_str(&format!("[dropbox] beacon: version={:?}, \
@@ -370,15 +406,15 @@ impl Format {
                                         dropbox.namespaces,
                                         dropbox.displayname,
                                         dropbox.port));
-                Purple
+                Color::Purple
             },
             Text(text) => {
                 out.push_str(&format!("[text] {:?}", text));
-                Red
+                Color::Red
             },
             Binary(x) => {
                 out.push_str(&format!("[binary] {:?}", x.as_bstr()));
-                Red
+                Color::Red
             },
         }
     }
@@ -423,66 +459,65 @@ impl Format {
 
         out.push_str(&format!("] {:?}", icmp.data.as_bstr()));
 
-        Blue
+        Color::Blue
     }
 
     #[inline]
     fn print_debugging(&self, packet: Raw) {
-        use crate::structs::raw::Raw::Unknown;
         match packet {
-            Ether(eth_frame, eth) => {
+            Raw::Ether(eth_frame, eth) => {
                 println!("eth: {:?}", eth_frame);
                 self.print_debugging_eth(1, eth);
             },
-            Tun(eth) => self.print_debugging_eth(0, eth),
-            Sll(eth) => self.print_debugging_eth(0, eth),
-            Unknown(data) => println!("unknown: {:?}", data),
+            Raw::Tun(eth) => self.print_debugging_eth(0, eth),
+            Raw::Sll(eth) => self.print_debugging_eth(0, eth),
+            Raw::Unknown(data) => println!("unknown: {:?}", data),
         }
     }
 
     #[inline]
-    fn print_debugging_eth(&self, indent: usize, eth: ether::Ether) {
+    fn print_debugging_eth(&self, indent: usize, eth: Ether) {
         match eth {
-            Arp(arp_pkt) => {
-                println!("{}{}", "\t".repeat(indent), self.colorify(Blue, format!("arp: {:?}", arp_pkt)));
+            Ether::Arp(arp_pkt) => {
+                println!("{}{}", "\t".repeat(indent), self.colorify(Color::Blue, format!("arp: {:?}", arp_pkt)));
             },
-            IPv4(ip_hdr, ipv4::IPv4::TCP(tcp_hdr, tcp)) => {
+            Ether::IPv4(ip_hdr, ipv4::IPv4::TCP(tcp_hdr, tcp)) => {
                 println!("{}ipv4: {:?}", "\t".repeat(indent), ip_hdr);
                 println!("{}tcp: {:?}",  "\t".repeat(indent+1), tcp_hdr);
                 println!("{}{}",         "\t".repeat(indent+2), self.print_debugging_tcp(tcp));
             },
-            IPv4(ip_hdr, ipv4::IPv4::UDP(udp_hdr, udp)) => {
+            Ether::IPv4(ip_hdr, ipv4::IPv4::UDP(udp_hdr, udp)) => {
                 println!("{}ipv4: {:?}", "\t".repeat(indent), ip_hdr);
                 println!("{}udp: {:?}",  "\t".repeat(indent+1), udp_hdr);
                 println!("{}{}",         "\t".repeat(indent+2), self.print_debugging_udp(udp));
             },
-            IPv4(ip_hdr, ipv4::IPv4::ICMP(icmp_hdr, icmp)) => {
+            Ether::IPv4(ip_hdr, ipv4::IPv4::ICMP(icmp_hdr, icmp)) => {
                 println!("{}ipv4: {:?}", "\t".repeat(indent), ip_hdr);
                 println!("{}icmp: {:?}",  "\t".repeat(indent+1), icmp_hdr);
                 println!("{}{:?}",         "\t".repeat(indent+2), icmp.data);
             },
-            IPv4(ip_hdr, ipv4::IPv4::Unknown(data)) => {
+            Ether::IPv4(ip_hdr, ipv4::IPv4::Unknown(data)) => {
                 println!("{}ipv4: {:?}",     "\t".repeat(indent), ip_hdr);
                 println!("{}unknown: {:?}",  "\t".repeat(indent+1), data);
             },
-            IPv6(ip_hdr, ipv6::IPv6::TCP(tcp_hdr, tcp)) => {
+            Ether::IPv6(ip_hdr, ipv6::IPv6::TCP(tcp_hdr, tcp)) => {
                 println!("{}ipv6: {:?}", "\t".repeat(indent), ip_hdr);
                 println!("{}tcp: {:?}",  "\t".repeat(indent+1), tcp_hdr);
                 println!("{}{}",         "\t".repeat(indent+2), self.print_debugging_tcp(tcp));
             },
-            IPv6(ip_hdr, ipv6::IPv6::UDP(udp_hdr, udp)) => {
+            Ether::IPv6(ip_hdr, ipv6::IPv6::UDP(udp_hdr, udp)) => {
                 println!("{}ipv6: {:?}", "\t".repeat(indent), ip_hdr);
                 println!("{}udp: {:?}",  "\t".repeat(indent+1), udp_hdr);
                 println!("{}{}",         "\t".repeat(indent+2), self.print_debugging_udp(udp));
             },
-            IPv6(ip_hdr, ipv6::IPv6::Unknown(data)) => {
+            Ether::IPv6(ip_hdr, ipv6::IPv6::Unknown(data)) => {
                 println!("{}ipv6: {:?}",     "\t".repeat(indent), ip_hdr);
                 println!("{}unknown: {:?}",  "\t".repeat(indent+1), data);
             },
-            Cjdns(cjdns_pkt) => {
+            Ether::Cjdns(cjdns_pkt) => {
                 println!("{}cjdns: {:?}",     "\t".repeat(indent), cjdns_pkt);
             },
-            ether::Ether::Unknown(data) => {
+            Ether::Unknown(data) => {
                 println!("{}unknown: {:?}", "\t".repeat(indent), data);
             }
         }
@@ -492,19 +527,22 @@ impl Format {
     fn print_debugging_tcp(&self, tcp: tcp::TCP) -> String {
         use crate::structs::tcp::TCP::*;
         match tcp {
-            HTTP(http) => {
-                self.colorify(Green, format!("http: {:?} {:?}", format!("{} http://{}{} HTTP/{}", http.method, http.host.clone().unwrap_or_else(|| "???".to_string()), http.uri, http.version), http))
+            HTTP(http::Http::Request(http)) => {
+                self.colorify(Color::Red, format!("http: {http:?}"))
+            },
+            HTTP(http::Http::Response(http)) => {
+                self.colorify(Color::Red, format!("http: {http:?}"))
             },
             TLS(client_hello) => {
-                self.colorify(Green, format!("tls: {:?}", client_hello))
+                self.colorify(Color::Green, format!("tls: {:?}", client_hello))
             },
             Text(text) => {
-                self.colorify(Blue, format!("remaining: {:?}", text))
+                self.colorify(Color::Blue, format!("remaining: {:?}", text))
             },
             Binary(x) => {
-                self.colorify(Yellow, format!("remaining: {:?}", x))
+                self.colorify(Color::Yellow, format!("remaining: {:?}", x))
             },
-            Empty => self.colorify(Fixed(GREY), String::new()),
+            Empty => self.colorify(GREY, String::new()),
         }
     }
 
@@ -513,22 +551,22 @@ impl Format {
         use crate::structs::udp::UDP::*;
         match udp {
             DHCP(dhcp) => {
-                self.colorify(Green, format!("dhcp: {:?}", dhcp))
+                self.colorify(Color::Green, format!("dhcp: {:?}", dhcp))
             },
             DNS(dns) => {
-                self.colorify(Green, format!("dns: {:?}", dns))
+                self.colorify(Color::Green, format!("dns: {:?}", dns))
             },
             SSDP(ssdp) => {
-                self.colorify(Purple, format!("ssdp: {:?}", ssdp))
+                self.colorify(Color::Purple, format!("ssdp: {:?}", ssdp))
             },
             Dropbox(dropbox) => {
-                self.colorify(Purple, format!("dropbox: {:?}", dropbox))
+                self.colorify(Color::Purple, format!("dropbox: {:?}", dropbox))
             },
             Text(text) => {
-                self.colorify(Blue, format!("remaining: {:?}", text))
+                self.colorify(Color::Blue, format!("remaining: {:?}", text))
             },
             Binary(x) => {
-                self.colorify(Yellow, format!("remaining: {:?}", x))
+                self.colorify(Color::Yellow, format!("remaining: {:?}", x))
             },
         }
     }
